#include "BVQXPluginInterface.h"

#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif
#include <stdio.h>
#include <stdlib.h>

#ifndef OS_WIN32
#include <dlfcn.h>
#endif

#ifdef OS_WIN32
#define GETSYMADDRESS GetProcAddress
#else
#define GETSYMADDRESS dlsym
#endif

// general functions
GetVersionOfBrainVoyagerQXPrototype qxGetVersionOfBrainVoyagerQX;
GetBrainVoyagerQXPathPrototype qxGetBrainVoyagerQXPath;
GetPluginsPathPrototype qxGetPluginsPath;

// input / output
ShowLogPanePrototype qxShowLogPane; // v2.0
ShowLogTabPrototype qxShowLogTab;  // deprecated
LogTextPrototype qxLogText;
MessageBoxPrototype qxMessageBox;
YesNoMessageBoxPrototype qxYesNoMessageBox;
GetIntegerInputPrototype qxGetIntegerInput;
GetFloatInputPrototype qxGetFloatInput;
GetTextInputPrototype qxGetTextInput;
GetOpenFileNamePrototype qxGetOpenFileName;
GetSaveFileNamePrototype qxGetSaveFileName;

// Plotting
CloseAllPlotsPrototype qxCloseAllPlots;
PlotPointsPrototype qxPlotPoints;
AddPointsToPlotPrototype qxAddPointsToPlot;
PlotLinePrototype qxPlotLine;
AddLineToPlotPrototype qxAddLineToPlot;
PlotBarsPrototype qxPlotBars;
AddBarsToPlotPrototype qxAddBarsToPlot;
ShowPlotAtPositionPrototype qxShowPlotAtPosition;
ResizePlotPrototype qxResizePlot;
SetBarPlotXAxisCategoriesPrototype qxSetBarPlotXAxisCategories;
SetPlotXAxisRangePrototype qxSetPlotXAxisRange;
SetPlotYAxisRangePrototype qxSetPlotYAxisRange;
SetPlotXAxisTitlePrototype qxSetPlotXAxisTitle;
SetPlotYAxisTitlePrototype qxSetPlotYAxisTitle;
SetPlotXAxisPropertiesPrototype qxSetPlotXAxisProperties;
SetPlotYAxisPropertiesPrototype qxSetPlotYAxisProperties;
SetPlotTitlePrototype qxSetPlotTitle;
SetPlotMarkerSizeAndColorPrototype qxSetPlotMarkerSizeAndColor;
SetPlotLineSizeAndColorPrototype qxSetPlotLineSizeAndColor;
SetPlotBarColorsPrototype qxSetPlotBarColors;
SetPlotFontSizesPrototype qxSetPlotFontSizes;
ShowPlotLegendPrototype qxShowPlotLegend;
SetPlotColorThemePrototype qxSetPlotColorTheme;

// GUI
UpdateActiveWindowPrototype qxUpdateActiveWindow;
ShowBusyCursorPrototype qxShowBusyCursor;
StopBusyCursorPrototype qxStopBusyCursor;

// Docs
GetNumberOfDocsInWorkspacePrototype qxGetNumberOfDocsInWorkspace;  // v1.10.4
GetDocumentTypePrototype qxGetDocumentType;  // v1.10.4
GetDocumentTypeNamePrototype qxGetDocumentTypeName;  // v1.10.4
GetIDOfCurrentDocumentPrototype qxGetIDOfCurrentDocument;  // v1.10.4
SetAsCurrentDocumentPrototype qxSetAsCurrentDocument;  // v1.10.4
CalculateIAMatrixPrototype qxCalculateIAMatrix;  // v2.0
GetTRFParametersFromVolumeToolsDlgPrototype qxGetTRFParametersFromVolumeToolsDlg;  // v2.0

// FMR
GetHeaderOfCurrentFMRPrototype qxGetHeaderOfCurrentFMR;
SetHeaderOfCurrentFMRPrototype qxSetHeaderOfCurrentFMR;
GetSTCFloatDataOfCurrentFMRPrototype qxGetSTCFloatDataOfCurrentFMR; // v2.0
GetSTCIntDataOfCurrentFMRPrototype qxGetSTCIntDataOfCurrentFMR; // v2.0
GetSTCDataOfCurrentFMRPrototype qxGetSTCDataOfCurrentFMR; // deprecated
SaveFMRPrototype qxSaveFMR;
SaveFMRAndSTCPrototype qxSaveFMRAndSTC;
GetScannerPositionHeaderOfCurrentFMRPrototype qxGetScannerPositionHeaderOfCurrentFMR;  // v1.10.4
SetScannerPositionHeaderOfCurrentFMRPrototype qxSetScannerPositionHeaderOfCurrentFMR;  // v1.10.4

// DMR
GetHeaderOfCurrentDMRPrototype qxGetHeaderOfCurrentDMR;
SetHeaderOfCurrentDMRPrototype qxSetHeaderOfCurrentDMR;
GetDWIFloatDataOfCurrentDMRPrototype qxGetDWIFloatDataOfCurrentDMR; // v2.0
GetDWIIntDataOfCurrentDMRPrototype qxGetDWIIntDataOfCurrentDMR; // v2.0
GetDWIDataOfCurrentDMRPrototype qxGetDWIDataOfCurrentDMR; // deprecated
ConvertDWIFormatOfCurrentDMRPrototype qxConvertDWIFormatOfCurrentDMR;
SaveDMRPrototype qxSaveDMR;
SaveDMRAndDWIPrototype qxSaveDMRAndDWI;

GetMAPOfCurrentFMRPrototype qxGetMAPOfCurrentFMR;
SetMAPParametersOfCurrentFMRPrototype qxSetMAPParametersOfCurrentFMR;
DeleteMAPsOfCurrentFMRPrototype qxDeleteMAPsOfCurrentFMR;
LoadMAPsToCurrentFMRPrototype qxLoadMAPsToCurrentFMR;
SaveMAPsOfCurrentFMRPrototype qxSaveMAPsOfCurrentFMR;
CreateMAPForCurrentFMRPrototype qxCreateMAPForCurrentFMR;

// VMR
GetCurrentVMRPrototype qxGetCurrentVMR;
GetMainHeaderOfCurrentVMRPrototype qxGetMainHeaderOfCurrentVMR;
SetMainHeaderOfCurrentVMRPrototype qxSetMainHeaderOfCurrentVMR;
GetScannerPositionHeaderOfCurrentVMRPrototype qxGetScannerPositionHeaderOfCurrentVMR;
SetScannerPositionHeaderOfCurrentVMRPrototype qxSetScannerPositionHeaderOfCurrentVMR;
GetPastTransformationsHeaderOfCurrentVMRPrototype qxGetPastTransformationsHeaderOfCurrentVMR;
SetPastTransformationsHeaderOfCurrentVMRPrototype qxSetPastTransformationsHeaderOfCurrentVMR;
ReframeCurrentVMRPrototype qxReframeCurrentVMR;
GetCrossPositionOfCurrentVMRPrototype qxGetCrossPositionOfCurrentVMR;

// V16
GetV16OfCurrentVMRPrototype qxGetV16OfCurrentVMR;
DetachV16FromCurrentVMRPrototype qxDetachV16FromCurrentVMR;
LoadV16ToCurrentVMRPrototype qxLoadV16ToCurrentVMR;
SaveV16OfCurrentVMRPrototype qxSaveV16OfCurrentVMR;
ConvertV16ToVMRPrototype qxConvertV16ToVMR;

// nrVMP
GetNRVMPsOfCurrentVMRPrototype qxGetNRVMPsOfCurrentVMR;
GetNRVMPOfCurrentVMRPrototype qxGetNRVMPOfCurrentVMR;
SetNRVMPParametersOfCurrentVMRPrototype qxSetNRVMPParametersOfCurrentVMR;
LoadNRVMPsToCurrentVMRPrototype qxLoadNRVMPsToCurrentVMR;
AddNRVMPsToCurrentVMRPrototype qxAddNRVMPsToCurrentVMR;
SaveNRVMPsOfCurrentVMRPrototype qxSaveNRVMPsOfCurrentVMR;
CreateNRVMPsForCurrentVMRPrototype qxCreateNRVMPsForCurrentVMR;
DeleteNRVMPsOfCurrentVMRPrototype qxDeleteNRVMPsOfCurrentVMR;
DeleteNRVMPMapPrototype	qxDeleteNRVMPMap;
SetNRVMPsParamsDisplayRangePrototype qxSetNRVMPsParamsDisplayRange;

// arVMP
GetARVMPsOfCurrentVMRPrototype qxGetARVMPsOfCurrentVMR;
GetARVMPOfCurrentVMRPrototype qxGetARVMPOfCurrentVMR;
SetARVMPParametersOfCurrentVMRPrototype qxSetARVMPParametersOfCurrentVMR;
LoadARVMPsToCurrentVMRPrototype qxLoadARVMPsToCurrentVMR;
SaveARVMPsOfCurrentVMRPrototype qxSaveARVMPsOfCurrentVMR;
CreateARVMPsForCurrentVMRPrototype qxCreateARVMPsForCurrentVMR;
DeleteARVMPsOfCurrentVMRPrototype qxDeleteARVMPsOfCurrentVMR;

// VTC
LinkVTCToCurrentVMRPrototype qxLinkVTCToCurrentVMR;
CreateVTCForCurrentVMRPrototype qxCreateVTCForCurrentVMR;
GetHeaderOfVTCOfCurrentVMRPrototype qxGetHeaderOfVTCOfCurrentVMR; // v2.0
SetHeaderOfVTCOfCurrentVMRPrototype qxSetHeaderOfVTCOfCurrentVMR; // v2.0
SetVTCParametersOfCurrentVMRPrototype qxSetVTCParametersOfCurrentVMR; // deprecated
GetVTCFloatDataOfCurrentVMRPrototype qxGetVTCFloatDataOfCurrentVMR; // v2.0
GetVTCIntDataOfCurrentVMRPrototype qxGetVTCIntDataOfCurrentVMR; // v2.0
GetVTCOfCurrentVMRPrototype qxGetVTCOfCurrentVMR; // deprecated
SaveVTCOfCurrentVMRPrototype qxSaveVTCOfCurrentVMR;

// Multi-VTC
ClearMultiVTCAnalysisPrototype qxClearMultiVTCAnalysis;
AddVTCRefForMultiVTCAnalysisPrototype qxAddVTCRefForMultiVTCAnalysis;
GetVoxelTimeCourseOfVTCPrototype qxGetVoxelTimeCourseOfVTC;  // v2.0 param changed
GetVoxelTimeCourseOfVTCsPrototype qxGetVoxelTimeCourseOfVTCs; // v2.0 param changed
GetVoxelCoordsTimeCourseOfVTCPrototype qxGetVoxelCoordsTimeCourseOfVTC; // v2.0 param changed
GetVoxelCoordsTimeCourseOfVTCsPrototype qxGetVoxelCoordsTimeCourseOfVTCs; // v2.0 param changed

// GLM
GetGLMOfCurrentVMRPrototype qxGetGLMOfCurrentVMR;
GetGLMContrastsOfCurrentVMRPrototype qxGetGLMContrastsOfCurrentVMR;

// SRF
GetScenePrototype qxGetScene;
LoadSRFPrototype qxLoadSRF;
AddSRFPrototype qxAddSRF;
RemoveCurrentSRFPrototype qxRemoveCurrentSRF;
GetCurrentSRFPrototype qxGetCurrentSRF;
ReleaseSRFAccessPrototype qxReleaseSRFAccess;
SaveCurrentSRFPrototype qxSaveCurrentSRF;
SetCurrentSRFPrototype qxSetCurrentSRF;

// MTC
LinkMTCToCurrentSRFPrototype qxLinkMTCToCurrentSRF;
GetMTCOfCurrentSRFPrototype qxGetMTCOfCurrentSRF;
SetMTCParametersOfCurrentSRFPrototype qxSetMTCParametersOfCurrentSRF;
CreateMTCForCurrentSRFPrototype qxCreateMTCForCurrentSRF;
SaveMTCOfCurrentSRFPrototype qxSaveMTCOfCurrentSRF;

// Multi-MTC (file-based read access)
ClearMultiMTCRefsOfCurrentSRFPrototype qxClearMultiMTCRefsOfCurrentSRF;
AddMultiMTCRefToCurrentSRFPrototype qxAddMultiMTCRefToCurrentSRF;
GetVertexTimeCourseOfMTCPrototype qxGetVertexTimeCourseOfMTC;
GetVertexTimeCourseOfAllMTCsPrototype qxGetVertexTimeCourseOfAllMTCs;

// SMP
GetSMPsOfCurrentSRFPrototype qxGetSMPsOfCurrentSRF;
GetSMPOfCurrentSRFPrototype qxGetSMPOfCurrentSRF;
SetSMPParametersOfCurrentSRFPrototype qxSetSMPParametersOfCurrentSRF;
LoadSMPsToCurrentSRFPrototype qxLoadSMPsToCurrentSRF;
SaveSMPsOfCurrentSRFPrototype qxSaveSMPsOfCurrentSRF;
CreateSMPForCurrentSRFPrototype qxCreateSMPForCurrentSRF;
DeleteSMPsOfCurrentSRFPrototype qxDeleteSMPsOfCurrentSRF;
OverlaySMPsOnSRFPrototype qxOverlaySMPsOnSRF;

// SSM
GetSSMOfCurrentSRFPrototype qxGetSSMOfCurrentSRF;
LoadSSMToCurrentSRFPrototype qxLoadSSMToCurrentSRF;
SaveSSMOfCurrentSRFPrototype qxSaveSSMOfCurrentSRF;
CreateSSMForCurrentSRFPrototype qxCreateSSMForCurrentSRF;
DeleteSSMOfCurrentSRFPrototype qxDeleteSSMOfCurrentSRF;

// MSK
LoadMaskToCurrentVMRPrototype qxLoadMaskToCurrentVMR;
GetMaskOfCurrentVMRPrototype qxGetMaskOfCurrentVMR;

// PRT
ClearProtocolPrototype qxClearProtocol;
LoadProtocolPrototype qxLoadProtocol;
SaveProtocolPrototype qxSaveProtocol;
GetProtocolPrototype qxGetProtocol;
SetProtocolPrototype qxSetProtocol;
GetProtocolConditionPrototype qxGetProtocolCondition;
AddProtocolConditionPrototype qxAddProtocolCondition;
AddProtocolConditionIntervalPrototype qxAddProtocolConditionInterval;

// PRED, DM, HRF
ClearDesignMatrixPrototype qxClearDesignMatrix;
AddPredictorPrototype qxAddPredictor;
SetPredictorValuesFromConditionPrototype qxSetPredictorValuesFromCondition;
SetPredictorValuesFromConditionNamePrototype qxSetPredictorValuesFromConditionName;
ApplyTwoGammaHRFToPredictorPrototype qxApplyTwoGammaHRFToPredictor;
ApplyTwoGammaHRFToPredictorWithNamePrototype qxApplyTwoGammaHRFToPredictorWithName;
ApplyBoyntonHRFToPredictorPrototype qxApplyBoyntonHRFToPredictor;
ApplyBoyntonHRFToPredictorWithNamePrototype qxApplyBoyntonHRFToPredictorWithName;
GetPredictorPrototype qxGetPredictor;
GetPredictorByNamePrototype qxGetPredictorByName;
LoadSingleStudyGLMDesignMatrixPrototype qxLoadSingleStudyGLMDesignMatrix;
SaveSingleStudyGLMDesignMatrixPrototype qxSaveSingleStudyGLMDesignMatrix;

// VOI
LoadVOIsPrototype qxLoadVOIs;
SaveVOIsPrototype qxSaveVOIs;
GetVOIsPrototype qxGetVOIs;
GetVOIPrototype qxGetVOI;

// POI
LoadPOIsPrototype qxLoadPOIs;
SavePOIsPrototype qxSavePOIs;
GetPOIsPrototype qxGetPOIs;
GetPOIPrototype qxGetPOI;

// get/set parameters between GUI plugin and GUI script
GetStringParameterPrototype qxGetStringParameter;
GetIntParameterPrototype qxGetIntParameter;
GetFloatParameterPrototype qxGetFloatParameter;
SetStringParameterPrototype qxSetStringParameter;
SetIntParameterPrototype qxSetIntParameter;
SetFloatParameterPrototype qxSetFloatParameter;

// dialog access - for internal use only (will be removed in a later API version)
GetDlgStringParameterPrototype qxGetDlgStringParameter;
GetDlgIntegerParameterPrototype qxGetDlgIntegerParameter;
GetDlgFloatParameterPrototype qxGetDlgFloatParameter;
SetDlgStringOutputParameterPrototype qxSetDlgStringOutputParameter;
SetDlgIntegerOutputParameterPrototype qxSetDlgIntegerOutputParameter;
SetDlgFloatOutputParameterPrototype qxSetDlgFloatOutputParameter;
LaunchICADialogPrototype qxLaunchICADialog;


bool InitQXAccess()
{
#ifdef OS_WIN32

	HMODULE hndl = GetModuleHandleA("BrainVoyagerQX.exe");

	if(hndl == 0)
		return false;

#else

	// handle Mac and Linux
	
	void *hndl = dlopen(NULL, RTLD_LAZY);  // NULL pointer to get handle to main process
	if(hndl == NULL)
		return false;
	
#endif

	//
	// general functions
	//
	qxGetVersionOfBrainVoyagerQX = (GetVersionOfBrainVoyagerQXPrototype)GETSYMADDRESS( hndl, "qxGetVersionOfBrainVoyagerQX" );
	if(qxGetVersionOfBrainVoyagerQX == NULL) return false;

	// IMPORTANT: If we do not get a pointer, we do not return false
	//  To use specific commands, a) you must check the obtained version string and use only those commands available in that version
	//                         or b) Check whether the function pointers are non-NULL for each command you want to use in your code

	qxGetBrainVoyagerQXPath = (GetBrainVoyagerQXPathPrototype)GETSYMADDRESS( hndl, "qxGetBrainVoyagerQXPath" );  // if(qxGetBrainVoyagerQXPath == NULL) return false;
	qxGetPluginsPath = (GetPluginsPathPrototype)GETSYMADDRESS( hndl, "qxGetPluginsPath" );

	//
	// input and display functions
	//
	qxShowLogPane = (ShowLogPanePrototype)GETSYMADDRESS( hndl, "qxShowLogPane" );  // if(qxShowLogPane == NULL) return false;
	qxShowLogTab = (ShowLogTabPrototype)GETSYMADDRESS( hndl, "qxShowLogTab" );
	qxLogText = (LogTextPrototype)GETSYMADDRESS( hndl, "qxLogText" );
	qxMessageBox = (MessageBoxPrototype)GETSYMADDRESS( hndl, "qxMessageBox" );
	qxYesNoMessageBox = (YesNoMessageBoxPrototype)GETSYMADDRESS( hndl, "qxYesNoMessageBox" );
	qxGetIntegerInput = (GetIntegerInputPrototype)GETSYMADDRESS( hndl, "qxGetIntegerInput" );
	qxGetFloatInput = (GetFloatInputPrototype)GETSYMADDRESS( hndl, "qxGetFloatInput" );
	qxGetTextInput = (GetTextInputPrototype)GETSYMADDRESS( hndl, "qxGetTextInput" );
	qxGetOpenFileName = (GetOpenFileNamePrototype)GETSYMADDRESS( hndl, "qxGetOpenFileName" );
	qxGetSaveFileName = (GetSaveFileNamePrototype)GETSYMADDRESS( hndl, "qxGetSaveFileName" );
	qxUpdateActiveWindow = (UpdateActiveWindowPrototype)GETSYMADDRESS( hndl, "qxUpdateActiveWindow" );
	qxShowBusyCursor = (ShowBusyCursorPrototype)GETSYMADDRESS( hndl, "qxShowBusyCursor" );
	qxStopBusyCursor = (StopBusyCursorPrototype)GETSYMADDRESS( hndl, "qxStopBusyCursor" );

    // Plotting
    qxCloseAllPlots = (CloseAllPlotsPrototype)GETSYMADDRESS( hndl, "qxCloseAllPlots");
	qxPlotPoints = (PlotPointsPrototype)GETSYMADDRESS( hndl, "qxPlotPoints");
    qxAddPointsToPlot = (AddPointsToPlotPrototype)GETSYMADDRESS( hndl, "qxAddPointsToPlot");
    qxPlotLine = (PlotLinePrototype)GETSYMADDRESS( hndl, "qxPlotLine");
    qxAddLineToPlot = (AddLineToPlotPrototype)GETSYMADDRESS( hndl, "qxAddLineToPlot");
    qxPlotBars = (PlotBarsPrototype)GETSYMADDRESS( hndl, "qxPlotBars");
    qxAddBarsToPlot = (AddBarsToPlotPrototype)GETSYMADDRESS( hndl, "qxAddBarsToPlot");
	qxShowPlotAtPosition = (ShowPlotAtPositionPrototype)GETSYMADDRESS( hndl, "qxShowPlotAtPosition");
	qxResizePlot = (ResizePlotPrototype)GETSYMADDRESS( hndl, "qxResizePlot");
    qxSetBarPlotXAxisCategories = (SetBarPlotXAxisCategoriesPrototype)GETSYMADDRESS( hndl, "qxSetBarPlotXAxisCategories");
    qxSetPlotXAxisRange = (SetPlotXAxisRangePrototype)GETSYMADDRESS( hndl, "qxSetPlotXAxisRange");
    qxSetPlotYAxisRange = (SetPlotYAxisRangePrototype)GETSYMADDRESS( hndl, "qxSetPlotYAxisRange");
    qxSetPlotXAxisTitle = (SetPlotXAxisTitlePrototype)GETSYMADDRESS( hndl, "qxSetPlotXAxisTitle");
	qxSetPlotYAxisTitle = (SetPlotYAxisTitlePrototype)GETSYMADDRESS( hndl, "qxSetPlotYAxisTitle");
    qxSetPlotXAxisProperties = (SetPlotXAxisPropertiesPrototype)GETSYMADDRESS( hndl, "qxSetPlotXAxisProperties");
    qxSetPlotYAxisProperties = (SetPlotYAxisPropertiesPrototype)GETSYMADDRESS( hndl, "qxSetPlotYAxisProperties");
    qxSetPlotTitle = (SetPlotTitlePrototype)GETSYMADDRESS( hndl, "qxSetPlotTitle");
	qxSetPlotMarkerSizeAndColor = (SetPlotMarkerSizeAndColorPrototype)GETSYMADDRESS( hndl, "qxSetPlotMarkerSizeAndColor");
	qxSetPlotLineSizeAndColor = (SetPlotLineSizeAndColorPrototype)GETSYMADDRESS( hndl, "qxSetPlotLineSizeAndColor");
	qxSetPlotBarColors = (SetPlotBarColorsPrototype)GETSYMADDRESS( hndl, "qxSetPlotBarColors");
	qxSetPlotFontSizes = (SetPlotFontSizesPrototype)GETSYMADDRESS( hndl, "qxSetPlotFontSizes");
	qxShowPlotLegend = (ShowPlotLegendPrototype)GETSYMADDRESS( hndl, "qxShowPlotLegend");
	qxSetPlotColorTheme = (SetPlotColorThemePrototype)GETSYMADDRESS( hndl, "qxSetPlotColorTheme");

	// Docs
	qxGetNumberOfDocsInWorkspace = (GetNumberOfDocsInWorkspacePrototype)GETSYMADDRESS( hndl, "qxGetNumberOfDocsInWorkspace" );
	qxGetDocumentType = (GetDocumentTypePrototype)GETSYMADDRESS( hndl, "qxGetDocumentType" );
	qxGetDocumentTypeName = (GetDocumentTypeNamePrototype)GETSYMADDRESS( hndl, "qxGetDocumentTypeName" );
	qxGetIDOfCurrentDocument = (GetIDOfCurrentDocumentPrototype)GETSYMADDRESS( hndl, "qxGetIDOfCurrentDocument" );
	qxSetAsCurrentDocument = (SetAsCurrentDocumentPrototype)GETSYMADDRESS( hndl, "qxSetAsCurrentDocument" );
	qxCalculateIAMatrix = (CalculateIAMatrixPrototype)GETSYMADDRESS( hndl, "qxCalculateIAMatrix" );
	qxGetTRFParametersFromVolumeToolsDlg = (GetTRFParametersFromVolumeToolsDlgPrototype)GETSYMADDRESS( hndl, "qxGetTRFParametersFromVolumeToolsDlg" );

	// FMR
	qxGetHeaderOfCurrentFMR = (GetHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetHeaderOfCurrentFMR" );
	qxSetHeaderOfCurrentFMR = (SetHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxSetHeaderOfCurrentFMR" );
	qxGetSTCFloatDataOfCurrentFMR = (GetSTCFloatDataOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetSTCFloatDataOfCurrentFMR" );
	qxGetSTCIntDataOfCurrentFMR = (GetSTCIntDataOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetSTCIntDataOfCurrentFMR" );
	qxGetSTCDataOfCurrentFMR = (GetSTCDataOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetSTCDataOfCurrentFMR" );
	qxSaveFMR = (SaveFMRPrototype)GETSYMADDRESS( hndl, "qxSaveFMR" );
	qxSaveFMRAndSTC = (SaveFMRAndSTCPrototype)GETSYMADDRESS( hndl, "qxSaveFMRAndSTC" );
	qxGetScannerPositionHeaderOfCurrentFMR = (GetScannerPositionHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetScannerPositionHeaderOfCurrentFMR" );
	qxSetScannerPositionHeaderOfCurrentFMR = (SetScannerPositionHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxSetScannerPositionHeaderOfCurrentFMR" );

	// DMR
	qxGetHeaderOfCurrentDMR = (GetHeaderOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxGetHeaderOfCurrentDMR" );
	qxSetHeaderOfCurrentDMR = (SetHeaderOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxSetHeaderOfCurrentDMR" );
	qxGetDWIFloatDataOfCurrentDMR = (GetDWIFloatDataOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxGetDWIFloatDataOfCurrentDMR" );
	qxGetDWIIntDataOfCurrentDMR = (GetDWIIntDataOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxGetDWIIntDataOfCurrentDMR" );
	qxGetDWIDataOfCurrentDMR = (GetDWIDataOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxGetDWIDataOfCurrentDMR" );
	qxConvertDWIFormatOfCurrentDMR = (ConvertDWIFormatOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxConvertDWIFormatOfCurrentDMR" );
	qxSaveDMR = (SaveDMRPrototype)GETSYMADDRESS( hndl, "qxSaveDMR" );
	qxSaveDMRAndDWI = (SaveDMRAndDWIPrototype)GETSYMADDRESS( hndl, "qxSaveDMRAndDWI" );

    // MAP
    qxGetMAPOfCurrentFMR = (GetMAPOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetMAPOfCurrentFMR" );
    qxSetMAPParametersOfCurrentFMR = (SetMAPParametersOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxSetMAPParametersOfCurrentFMR");
    qxDeleteMAPsOfCurrentFMR = (DeleteMAPsOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxDeleteMAPsOfCurrentFMR");
    qxLoadMAPsToCurrentFMR = (LoadMAPsToCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxLoadMAPsToCurrentFMR");
    qxSaveMAPsOfCurrentFMR = (SaveMAPsOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxSaveMAPsOfCurrentFMR");
    qxCreateMAPForCurrentFMR = (CreateMAPForCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxCreateMAPForCurrentFMR" );

	// VMR
	qxGetCurrentVMR = (GetCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetCurrentVMR" );
	qxGetMainHeaderOfCurrentVMR = (GetMainHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetMainHeaderOfCurrentVMR" );
	qxSetMainHeaderOfCurrentVMR = (SetMainHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetMainHeaderOfCurrentVMR" );
	qxGetScannerPositionHeaderOfCurrentVMR = (GetScannerPositionHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetScannerPositionHeaderOfCurrentVMR" );
	qxSetScannerPositionHeaderOfCurrentVMR = (SetScannerPositionHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetScannerPositionHeaderOfCurrentVMR" );
	qxGetPastTransformationsHeaderOfCurrentVMR = (GetPastTransformationsHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetPastTransformationsHeaderOfCurrentVMR" );
	qxSetPastTransformationsHeaderOfCurrentVMR = (SetPastTransformationsHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetPastTransformationsHeaderOfCurrentVMR" );
	qxReframeCurrentVMR = (ReframeCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxReframeCurrentVMR" );
	qxGetCrossPositionOfCurrentVMR = (GetCrossPositionOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetCrossPositionOfCurrentVMR" );

	// V16
	qxGetV16OfCurrentVMR = (GetV16OfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetV16OfCurrentVMR" );
    qxLoadV16ToCurrentVMR = (LoadV16ToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLoadV16ToCurrentVMR" );
    qxSaveV16OfCurrentVMR = (SaveV16OfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSaveV16OfCurrentVMR" );
    qxDetachV16FromCurrentVMR = (DetachV16FromCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxDetachV16FromCurrentVMR" );
    qxConvertV16ToVMR = (ConvertV16ToVMRPrototype)GETSYMADDRESS( hndl, "qxConvertV16ToVMR" );
    
	// nrVMP
	qxGetNRVMPsOfCurrentVMR = (GetNRVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetNRVMPsOfCurrentVMR" );
	qxGetNRVMPOfCurrentVMR = (GetNRVMPOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetNRVMPOfCurrentVMR" );
	qxSetNRVMPParametersOfCurrentVMR = (SetNRVMPParametersOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetNRVMPParametersOfCurrentVMR" );
	qxLoadNRVMPsToCurrentVMR = (LoadNRVMPsToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLoadNRVMPsToCurrentVMR" );
	qxAddNRVMPsToCurrentVMR = (AddNRVMPsToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxAddNRVMPsToCurrentVMR" );
	qxSaveNRVMPsOfCurrentVMR = (SaveNRVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSaveNRVMPsOfCurrentVMR" );
	qxCreateNRVMPsForCurrentVMR = (CreateNRVMPsForCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxCreateNRVMPsForCurrentVMR" );
	qxDeleteNRVMPsOfCurrentVMR = (DeleteNRVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxDeleteNRVMPsOfCurrentVMR" );
	qxDeleteNRVMPMap = (DeleteNRVMPMapPrototype)GETSYMADDRESS( hndl, "qxDeleteNRVMPMap" );
	qxSetNRVMPsParamsDisplayRange = (SetNRVMPsParamsDisplayRangePrototype)GETSYMADDRESS( hndl, "qxSetNRVMPsParamsDisplayRange" );

	// arVMP
	qxGetARVMPsOfCurrentVMR = (GetARVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetARVMPsOfCurrentVMR" );
	qxGetARVMPOfCurrentVMR = (GetARVMPOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetARVMPOfCurrentVMR" );
	qxSetARVMPParametersOfCurrentVMR = (SetARVMPParametersOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetARVMPParametersOfCurrentVMR" );
	qxLoadARVMPsToCurrentVMR = (LoadARVMPsToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLoadARVMPsToCurrentVMR" );
	qxSaveARVMPsOfCurrentVMR = (SaveARVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSaveARVMPsOfCurrentVMR" );
	qxCreateARVMPsForCurrentVMR = (CreateARVMPsForCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxCreateARVMPsForCurrentVMR" );
	qxDeleteARVMPsOfCurrentVMR = (DeleteARVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxDeleteARVMPsOfCurrentVMR" );

	// VTC
	qxLinkVTCToCurrentVMR = (LinkVTCToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLinkVTCToCurrentVMR" );
	qxCreateVTCForCurrentVMR = (CreateVTCForCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxCreateVTCForCurrentVMR" );
	qxGetHeaderOfVTCOfCurrentVMR = (GetHeaderOfVTCOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetHeaderOfVTCOfCurrentVMR" );
	qxSetHeaderOfVTCOfCurrentVMR = (SetHeaderOfVTCOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetHeaderOfVTCOfCurrentVMR" );
	qxGetVTCOfCurrentVMR = (GetVTCOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetVTCOfCurrentVMR" );
	qxGetVTCFloatDataOfCurrentVMR = (GetVTCFloatDataOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetVTCFloatDataOfCurrentVMR" );
	qxGetVTCIntDataOfCurrentVMR = (GetVTCIntDataOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetVTCIntDataOfCurrentVMR" );
	qxSetVTCParametersOfCurrentVMR = (SetVTCParametersOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetVTCParametersOfCurrentVMR" );
	qxSaveVTCOfCurrentVMR = (SaveVTCOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSaveVTCOfCurrentVMR" );

	// Multi-VTC
	qxClearMultiVTCAnalysis = (ClearMultiVTCAnalysisPrototype)GETSYMADDRESS( hndl, "qxClearMultiVTCAnalysis" );
	qxAddVTCRefForMultiVTCAnalysis = (AddVTCRefForMultiVTCAnalysisPrototype)GETSYMADDRESS( hndl, "qxAddVTCRefForMultiVTCAnalysis" );
	qxGetVoxelTimeCourseOfVTC = (GetVoxelTimeCourseOfVTCPrototype)GETSYMADDRESS( hndl, "qxGetVoxelTimeCourseOfVTC" );
	qxGetVoxelTimeCourseOfVTCs = (GetVoxelTimeCourseOfVTCsPrototype)GETSYMADDRESS( hndl, "qxGetVoxelTimeCourseOfVTCs" );
	qxGetVoxelCoordsTimeCourseOfVTC = (GetVoxelCoordsTimeCourseOfVTCPrototype)GETSYMADDRESS( hndl, "qxGetVoxelCoordsTimeCourseOfVTC" );
	qxGetVoxelCoordsTimeCourseOfVTCs = (GetVoxelCoordsTimeCourseOfVTCsPrototype)GETSYMADDRESS( hndl, "qxGetVoxelCoordsTimeCourseOfVTCs" );

	// GLM
	qxGetGLMOfCurrentVMR = (GetGLMOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetGLMOfCurrentVMR" );
	qxGetGLMContrastsOfCurrentVMR = (GetGLMContrastsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetGLMContrastsOfCurrentVMR" );

	// MASK
	qxLoadMaskToCurrentVMR = (LoadMaskToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLoadMaskToCurrentVMR" );
	qxGetMaskOfCurrentVMR = (GetMaskOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetMaskOfCurrentVMR" );

	// SRF
	qxGetScene = (GetScenePrototype)GETSYMADDRESS( hndl, "qxGetScene" );
    qxLoadSRF = (LoadSRFPrototype)GETSYMADDRESS( hndl, "qxLoadSRF" );
    qxAddSRF = (LoadSRFPrototype)GETSYMADDRESS( hndl, "qxAddSRF" );
    qxRemoveCurrentSRF = (RemoveCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxRemoveCurrentSRF" );
	qxGetCurrentSRF = (GetCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetCurrentSRF" );
	qxReleaseSRFAccess = (ReleaseSRFAccessPrototype)GETSYMADDRESS( hndl, "qxReleaseSRFAccess" );
	qxSaveCurrentSRF = (SaveCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveCurrentSRF" );
	qxSetCurrentSRF = (SetCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSetCurrentSRF" );

	// MTC
	qxLinkMTCToCurrentSRF = (LinkMTCToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxLinkMTCToCurrentSRF" );
	qxGetMTCOfCurrentSRF = (GetMTCOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetMTCOfCurrentSRF" );
	qxSetMTCParametersOfCurrentSRF = (SetMTCParametersOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSetMTCParametersOfCurrentSRF" );
	qxCreateMTCForCurrentSRF = (CreateMTCForCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxCreateMTCForCurrentSRF" );
	qxSaveMTCOfCurrentSRF = (SaveMTCOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveMTCOfCurrentSRF" );

	// Multi-MTC
	qxClearMultiMTCRefsOfCurrentSRF = (ClearMultiMTCRefsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxClearMultiMTCRefsOfCurrentSRF" );
	qxAddMultiMTCRefToCurrentSRF = (AddMultiMTCRefToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxAddMultiMTCRefToCurrentSRF" );
	qxGetVertexTimeCourseOfMTC = (GetVertexTimeCourseOfMTCPrototype)GETSYMADDRESS( hndl, "qxGetVertexTimeCourseOfMTC" );
	qxGetVertexTimeCourseOfAllMTCs = (GetVertexTimeCourseOfAllMTCsPrototype)GETSYMADDRESS( hndl, "qxGetVertexTimeCourseOfAllMTCs" );

	// SMP
	qxGetSMPsOfCurrentSRF = (GetSMPsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetSMPsOfCurrentSRF" );
	qxGetSMPOfCurrentSRF = (GetSMPOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetSMPOfCurrentSRF" );
	qxSetSMPParametersOfCurrentSRF = (SetSMPParametersOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSetSMPParametersOfCurrentSRF" );
	qxLoadSMPsToCurrentSRF = (LoadSMPsToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxLoadSMPsToCurrentSRF" );
	qxSaveSMPsOfCurrentSRF = (SaveSMPsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveSMPsOfCurrentSRF" );
	qxCreateSMPForCurrentSRF = (CreateSMPForCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxCreateSMPForCurrentSRF" );
	qxDeleteSMPsOfCurrentSRF = (DeleteSMPsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxDeleteSMPsOfCurrentSRF" );
	qxOverlaySMPsOnSRF = (OverlaySMPsOnSRFPrototype)GETSYMADDRESS( hndl, "qxOverlaySMPsOnSRF" );

	// SSM
	qxGetSSMOfCurrentSRF = (GetSSMOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetSSMOfCurrentSRF" );
	qxLoadSSMToCurrentSRF = (LoadSSMToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxLoadSSMToCurrentSRF" );
	qxSaveSSMOfCurrentSRF = (SaveSSMOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveSSMOfCurrentSRF" );
	qxCreateSSMForCurrentSRF = (CreateSSMForCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxCreateSSMForCurrentSRF" );
	qxDeleteSSMOfCurrentSRF = (DeleteSSMOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxDeleteSSMOfCurrentSRF" );

	// PRT
	qxClearProtocol = (ClearProtocolPrototype)GETSYMADDRESS( hndl, "qxClearProtocol" );
	qxLoadProtocol = (LoadProtocolPrototype)GETSYMADDRESS( hndl, "qxLoadProtocol" );
	qxSaveProtocol = (SaveProtocolPrototype)GETSYMADDRESS( hndl, "qxSaveProtocol" );
	qxGetProtocol = (GetProtocolPrototype)GETSYMADDRESS( hndl, "qxGetProtocol" );
	qxSetProtocol = (SetProtocolPrototype)GETSYMADDRESS( hndl, "qxSetProtocol" );
	qxGetProtocolCondition = (GetProtocolConditionPrototype)GETSYMADDRESS( hndl, "qxGetProtocolCondition" );
	qxAddProtocolCondition = (AddProtocolConditionPrototype)GETSYMADDRESS( hndl, "qxAddProtocolCondition" );
	qxAddProtocolConditionInterval = (AddProtocolConditionIntervalPrototype)GETSYMADDRESS( hndl, "qxAddProtocolConditionInterval" );

	// Predictor and design matrix modelling functions
	qxClearDesignMatrix = (ClearDesignMatrixPrototype)GETSYMADDRESS( hndl, "qxClearDesignMatrix" );
	qxAddPredictor = (AddPredictorPrototype)GETSYMADDRESS( hndl, "qxAddPredictor" );
	qxSetPredictorValuesFromCondition = (SetPredictorValuesFromConditionPrototype)GETSYMADDRESS( hndl, "qxSetPredictorValuesFromCondition" );
	qxSetPredictorValuesFromConditionName = (SetPredictorValuesFromConditionNamePrototype)GETSYMADDRESS( hndl, "qxSetPredictorValuesFromConditionName" );
	qxApplyTwoGammaHRFToPredictor = (ApplyTwoGammaHRFToPredictorPrototype)GETSYMADDRESS( hndl, "qxApplyTwoGammaHRFToPredictor" );
	qxApplyTwoGammaHRFToPredictorWithName = (ApplyTwoGammaHRFToPredictorWithNamePrototype)GETSYMADDRESS( hndl, "qxApplyTwoGammaHRFToPredictorWithName" );
	qxApplyBoyntonHRFToPredictor = (ApplyBoyntonHRFToPredictorPrototype)GETSYMADDRESS( hndl, "qxApplyBoyntonHRFToPredictor" );
	qxApplyBoyntonHRFToPredictorWithName = (ApplyBoyntonHRFToPredictorWithNamePrototype)GETSYMADDRESS( hndl, "qxApplyBoyntonHRFToPredictorWithName" );
	qxGetPredictor = (GetPredictorPrototype)GETSYMADDRESS( hndl, "qxGetPredictor" );
	qxGetPredictorByName = (GetPredictorByNamePrototype)GETSYMADDRESS( hndl, "qxGetPredictorByName" );
	qxLoadSingleStudyGLMDesignMatrix = (LoadSingleStudyGLMDesignMatrixPrototype)GETSYMADDRESS( hndl, "qxLoadSingleStudyGLMDesignMatrix" );
	qxSaveSingleStudyGLMDesignMatrix = (SaveSingleStudyGLMDesignMatrixPrototype)GETSYMADDRESS( hndl, "qxSaveSingleStudyGLMDesignMatrix" );

	// VOI
	qxLoadVOIs = (LoadVOIsPrototype)GETSYMADDRESS( hndl, "qxLoadVOIs" );
	qxSaveVOIs = (SaveVOIsPrototype)GETSYMADDRESS( hndl, "qxSaveVOIs" );
	qxGetVOIs = (GetVOIsPrototype)GETSYMADDRESS( hndl, "qxGetVOIs" );
	qxGetVOI  = (GetVOIPrototype)GETSYMADDRESS( hndl, "qxGetVOI" );

	// POI
	qxLoadPOIs = (LoadPOIsPrototype)GETSYMADDRESS( hndl, "qxLoadPOIs" );
	qxSavePOIs = (SavePOIsPrototype)GETSYMADDRESS( hndl, "qxSavePOIs" );
	qxGetPOIs = (GetPOIsPrototype)GETSYMADDRESS( hndl, "qxGetPOIs" );
	qxGetPOI  = (GetPOIPrototype)GETSYMADDRESS( hndl, "qxGetPOI" );

	// get/set parameters between GUI plugin and GUI script
	qxGetStringParameter = (GetStringParameterPrototype)GETSYMADDRESS( hndl, "qxGetStringParameter" );
	qxGetIntParameter    = (GetIntParameterPrototype)GETSYMADDRESS( hndl, "qxGetIntParameter" );
	qxGetFloatParameter  = (GetFloatParameterPrototype)GETSYMADDRESS( hndl, "qxGetFloatParameter" );
	qxSetStringParameter = (SetStringParameterPrototype)GETSYMADDRESS( hndl, "qxSetStringParameter" );
	qxSetIntParameter    = (SetIntParameterPrototype)GETSYMADDRESS( hndl, "qxSetIntParameter" );
	qxSetFloatParameter  = (SetFloatParameterPrototype)GETSYMADDRESS( hndl, "qxSetFloatParameter" );

	// get dlg parameters for special plugins with dialog provided by BrainVoyagerQX (deprecated, will be removed in future version)
	qxGetDlgStringParameter = (GetDlgStringParameterPrototype)GETSYMADDRESS( hndl, "qxGetDlgStringParameter" );
	qxGetDlgIntegerParameter = (GetDlgIntegerParameterPrototype)GETSYMADDRESS( hndl, "qxGetDlgIntegerParameter" );
	qxGetDlgFloatParameter = (GetDlgFloatParameterPrototype)GETSYMADDRESS( hndl, "qxGetDlgFloatParameter" );
	qxSetDlgStringOutputParameter = (SetDlgStringOutputParameterPrototype)GETSYMADDRESS( hndl, "qxSetDlgStringOutputParameter" );
	qxSetDlgIntegerOutputParameter = (SetDlgIntegerOutputParameterPrototype)GETSYMADDRESS( hndl, "qxSetDlgIntegerOutputParameter" );
	qxSetDlgFloatOutputParameter = (SetDlgFloatOutputParameterPrototype)GETSYMADDRESS( hndl, "qxSetDlgFloatOutputParameter" );
	qxLaunchICADialog = (LaunchICADialogPrototype)GETSYMADDRESS( hndl, "qxLaunchICADialog" );

	return true;
}

bool checkVersionGreaterEqualVersion(int MajorA, int MinorA, int SubMinorA, int MajorB, int MinorB, int SubMinorB)
{
	if(MajorA > MajorB)
		return true;

	if(MajorB > MajorA)
		return false;

	// Major is equal, Minor decides

	if(MinorA > MinorB)
		return true;

	if(MinorB > MinorA)
		return false;

	// Minor is equal, SubMinor decides

	if(SubMinorA >= SubMinorB)
		return true;

	// B smaller
	return false;
}
